#include "Config.h"
#include "Defines.h"

Config::Config()
: m_bRetroStyle(false), m_bSelectorMovementAnimate(true)
{
   ArchiveCreate(&m_Archive);

   OpenArchiveFile(m_Archive, "PuzzleFrenzySettings.tns");

   int nSettings = GetNumberArchives(m_Archive, "Settings");
   char strName[MAX_NAME_LENGTH];
   for(int i=0; i<nSettings; i++) {
      strcpy(strName, GetName(m_Archive, "Settings", i));

      if( strcmp(strName, "UseRetroStyle") == 0 ) {
         if( strcmp("0", GetValue(m_Archive, "Settings", i)) == 0 ) {
            m_bRetroStyle = false;
         } else {
            m_bRetroStyle = true;
         }
      }

      if( strcmp(strName, "SelectorMovementAnimate") == 0 ) {
         if( strcmp("0", GetValue(m_Archive, "Settings", i)) == 0 ) {
            m_bSelectorMovementAnimate = false;
         } else {
            m_bSelectorMovementAnimate = true;
         }
      }
   }
}

Config::~Config()
{
   ArchiveSetBatchMode(m_Archive, ARCHIVE_ENABLE_BATCH);
   UpdateArchiveEntry(m_Archive, "Settings", "UseRetroStyle", m_bRetroStyle ? "1" : "0", NULL);
   UpdateArchiveEntry(m_Archive, "Settings", "SelectorMovementAnimate", m_bSelectorMovementAnimate ? "1" : "0", NULL);
   ArchiveSetBatchMode(m_Archive, ARCHIVE_DISABLE_BATCH);

   ArchiveFree(&m_Archive);
}

bool Config::GetRetroStyle() const
{
   return m_bRetroStyle;
}

void Config::SetRetroStyle(bool bEnable)
{
   m_bRetroStyle = bEnable;
}

bool Config::GetSelectorMovementAnimate() const
{
   return m_bSelectorMovementAnimate;
}

void Config::SetSelectorMovementAnimate(bool bEnable)
{
   m_bSelectorMovementAnimate = bEnable;
}

bool Config::BeatLevel(int nLevelNumber, int& nCompletionTime) const
{
   char buffer[8];
   Puz_itoa(nLevelNumber, buffer, 8);

   int nLevelsBeat = GetNumberArchives(m_Archive, "LevelsCompleted");
   char strName[MAX_NAME_LENGTH];
   bool bCompleted = false;
   for(int i=0; i<nLevelsBeat; i++) {
      strcpy(strName, GetName(m_Archive, "LevelsCompleted", i));
      if( strcmp(strName, buffer) == 0 ) {
         if( strcmp("1", GetValue(m_Archive, "LevelsCompleted", i)) == 0 ) {
            bCompleted = true;
            break;
         } else {
            break;
         }
      }
   }

   int nTimeCompletions = GetNumberArchives(m_Archive, "CompletionTime");
   for(int i=0; i<nTimeCompletions; i++) {
      strcpy(strName, GetName(m_Archive, "CompletionTime", i));
      if( strcmp(strName, buffer) == 0 ) {
         nCompletionTime = atoi(GetValue(m_Archive, "CompletionTime", i));
         break;
      }
   }

   return bCompleted;
}

void Config::SetBeatLevel(int nLevelNumber, int nTimeElapsed)
{
   char buffer[8];
   Puz_itoa(nLevelNumber, buffer, 8);
   UpdateArchiveEntry(m_Archive, "LevelsCompleted", buffer, "1", NULL);

   bool bWriteCompletionTime = false;
   int nCompletionTime = -1;
   if( BeatLevel(nLevelNumber, nCompletionTime) ) {
      if( nTimeElapsed < nCompletionTime ) {
         bWriteCompletionTime = true;
      }
   }
   else {
      bWriteCompletionTime = true;
   }

   char buffer2[8];
   Puz_itoa(nTimeElapsed, buffer2, 8);
   UpdateArchiveEntry(m_Archive, "CompletionTime", buffer, buffer2, NULL);
}

